@@ -6,7 +6,12 @@ from typing import TYPE_CHECKING
 import pyarrow as pa
 import pyarrow.compute as pc
 
-from narwhals._arrow.utils import ArrowSeriesNamespace, lit, parse_datetime_format
+from narwhals._arrow.utils import (
+    ArrowSeriesNamespace,
+    extract_native,
+    lit,
+    parse_datetime_format,
+)
 from narwhals._compliant.any_namespace import StringNamespace
 
 if TYPE_CHECKING:
@@ -18,25 +23,23 @@ class ArrowSeriesStringNamespace(ArrowSeriesNamespace, StringNamespace["ArrowSer
     def len_chars(self) -> ArrowSeries:
         return self.with_native(pc.utf8_length(self.native))
 
-    def replace(self, pattern: str, value: str, *, literal: bool, n: int) -> ArrowSeries:
+    def replace(
+        self, value: ArrowSeries, pattern: str, *, literal: bool, n: int
+    ) -> ArrowSeries:
         fn = pc.replace_substring if literal else pc.replace_substring_regex
-        try:
-            arr = fn(self.native, pattern, replacement=value, max_replacements=n)
-        except TypeError as e:
-            if not isinstance(value, str):
-                msg = "PyArrow backed `.str.replace` only supports str replacement values"
-                raise TypeError(msg) from e
-            raise
+        _, value_native = extract_native(self.compliant, value)
+        if not isinstance(value_native, pa.StringScalar):
+            msg = "PyArrow backed `.str.replace` only supports str replacement values"
+            raise TypeError(msg)
+        arr = fn(
+            self.native, pattern, replacement=value_native.as_py(), max_replacements=n
+        )
         return self.with_native(arr)
 
-    def replace_all(self, pattern: str, value: str, *, literal: bool) -> ArrowSeries:
-        try:
-            return self.replace(pattern, value, literal=literal, n=-1)
-        except TypeError as e:
-            if not isinstance(value, str):
-                msg = "PyArrow backed `.str.replace_all` only supports str replacement values."
-                raise TypeError(msg) from e
-            raise
+    def replace_all(
+        self, value: ArrowSeries, pattern: str, *, literal: bool
+    ) -> ArrowSeries:
+        return self.replace(value, pattern, literal=literal, n=-1)
 
     def strip_chars(self, characters: str | None) -> ArrowSeries:
         return self.with_native(
