@@ -3,6 +3,7 @@ from __future__ import annotations
 from itertools import chain
 from typing import TYPE_CHECKING
 
+from narwhals._duckdb.utils import join_column_names
 from narwhals._sql.group_by import SQLGroupBy
 
 if TYPE_CHECKING:
@@ -27,7 +28,12 @@ class DuckDBGroupBy(SQLGroupBy["DuckDBLazyFrame", "DuckDBExpr", "Expression"]):
         self._compliant_frame = frame.drop_nulls(self._keys) if drop_null_keys else frame
 
     def agg(self, *exprs: DuckDBExpr) -> DuckDBLazyFrame:
-        agg_columns = list(chain(self._keys, self._evaluate_exprs(exprs)))
-        return self.compliant._with_native(
-            self.compliant.native.aggregate(agg_columns)  # type: ignore[arg-type]
-        ).rename(dict(zip(self._keys, self._output_key_names)))
+        agg_columns = tuple(self._evaluate_exprs(exprs))
+        result = self.compliant.native.aggregate(
+            tuple(chain(self._keys, agg_columns)),  # type: ignore[arg-type]
+            join_column_names(*self._keys),
+        )
+
+        return self.compliant._with_native(result).rename(
+            dict(zip(self._keys, self._output_key_names))
+        )
